@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -23,7 +23,6 @@ import enum
 from collections import namedtuple
 from typing import Optional, Callable, Tuple, TypeVar, Union, Any, Dict
 
-import pkg_resources
 from evdev.ecodes import (
     EV_KEY,
     EV_ABS,
@@ -33,6 +32,9 @@ from evdev.ecodes import (
     REL_HWHEEL_HI_RES,
     REL_WHEEL_HI_RES,
 )
+from packaging import version
+
+from inputremapper.logging.logger import logger
 
 try:
     from pydantic.v1 import (
@@ -62,7 +64,7 @@ except ImportError:
     )
 
 from inputremapper.configs.input_config import InputCombination
-from inputremapper.configs.system_mapping import system_mapping, DISABLE_NAME
+from inputremapper.configs.keyboard_layout import keyboard_layout, DISABLE_NAME
 from inputremapper.configs.validation_errors import (
     OutputSymbolUnknownError,
     SymbolNotAvailableInTargetError,
@@ -71,22 +73,20 @@ from inputremapper.configs.validation_errors import (
     OutputSymbolVariantError,
     MacroButTypeOrCodeSetError,
     SymbolAndCodeMismatchError,
-    MissingMacroOrKeyError,
+    WrongMappingTypeForKeyError,
     MissingOutputAxisError,
-    MacroParsingError,
+    MissingMacroOrKeyError,
 )
 from inputremapper.gui.gettext import _
 from inputremapper.gui.messages.message_types import MessageType
-from inputremapper.injection.global_uinputs import can_default_uinput_emit
-from inputremapper.injection.macros.parse import is_this_a_macro, parse
+from inputremapper.injection.global_uinputs import GlobalUInputs
+from inputremapper.injection.macros.parse import Parser
 from inputremapper.utils import get_evdev_constant_name
 
 # TODO: remove pydantic VERSION check as soon as we no longer support
 #  Ubuntu 20.04 and with it the ancient pydantic 1.2
 
-needs_workaround = pkg_resources.parse_version(
-    str(VERSION)
-) < pkg_resources.parse_version("1.7.1")
+needs_workaround = version.parse(str(VERSION)) < version.parse("1.7.1")
 
 
 EMPTY_MAPPING_NAME: str = _("Empty Mapping")
@@ -155,7 +155,9 @@ class UIMapping(BaseModel):
     target_uinput: Optional[Union[str, KnownUinput]] = None
 
     # Either `output_symbol` or `output_type` and `output_code` is required
+    # Only set if output is "Key or Macro":
     output_symbol: Optional[str] = None  # The symbol or macro string if applicable
+    # "Analog Axis" or if preset edited manually to inject a code instead of a symbol:
     output_type: Optional[int] = None  # The event type of the mapped event
     output_code: Optional[int] = None  # The event code of the mapped event
 
@@ -289,13 +291,15 @@ class UIMapping(BaseModel):
 
     def get_output_type_code(self) -> Optional[Tuple[int, int]]:
         """Returns the output_type and output_code if set,
-        otherwise looks the output_symbol up in the system_mapping
+        otherwise looks the output_symbol up in the keyboard_layout
         return None for unknown symbols and macros
         """
-        if self.output_code and self.output_type:
+        if self.output_code is not None and self.output_type is not None:
             return self.output_type, self.output_code
-        if self.output_symbol and not is_this_a_macro(self.output_symbol):
-            return EV_KEY, system_mapping.get(self.output_symbol)
+
+        if self.output_symbol and not Parser.is_this_a_macro(self.output_symbol):
+            return EV_KEY, keyboard_layout.get(self.output_symbol)
+
         return None
 
     def get_output_name_constant(self) -> str:
@@ -325,11 +329,20 @@ class UIMapping(BaseModel):
         output_code = values.get("output_code")
         output_symbol = values.get("output_symbol")
 
-        if output_type is not None and output_code is not None and not output_symbol:
-            values["mapping_type"] = "analog"
+        if output_type is not None and output_symbol is not None:
+            # This is currently only possible when someone edits the preset file by
+            # hand. A key-output mapping without an output_symbol, but type and code
+            # instead, is valid as well.
+            logger.debug("Both output_type and output_symbol are set")
+
+        if output_type != EV_KEY and output_code is not None and not output_symbol:
+            values["mapping_type"] = MappingType.ANALOG.value
 
         if output_type is None and output_code is None and output_symbol:
-            values["mapping_type"] = "key_macro"
+            values["mapping_type"] = MappingType.KEY_MACRO.value
+
+        if output_type == EV_KEY:
+            values["mapping_type"] = MappingType.KEY_MACRO.value
 
         return values
 
@@ -348,7 +361,10 @@ class Mapping(UIMapping):
 
     @classmethod
     def from_combination(
-        cls, input_combination=None, target_uinput="keyboard", output_symbol="a"
+        cls,
+        input_combination=None,
+        target_uinput="keyboard",
+        output_symbol="a",
     ):
         """Convenient function to get a valid mapping."""
         if not input_combination:
@@ -382,18 +398,20 @@ class Mapping(UIMapping):
         if symbol == DISABLE_NAME:
             return values
 
-        if is_this_a_macro(symbol):
+        if Parser.is_this_a_macro(symbol):
             mapping_mock = namedtuple("Mapping", values.keys())(**values)
-            # raises MacroParsingError
-            parse(symbol, mapping=mapping_mock, verbose=False)
+            # raises MacroError
+            Parser.parse(symbol, mapping=mapping_mock, verbose=False)
             return values
 
-        code = system_mapping.get(symbol)
+        code = keyboard_layout.get(symbol)
         if code is None:
             raise OutputSymbolUnknownError(symbol)
 
         target = values.get("target_uinput")
-        if target is not None and not can_default_uinput_emit(target, EV_KEY, code):
+        if target is not None and not GlobalUInputs.can_default_uinput_emit(
+            target, EV_KEY, code
+        ):
             raise SymbolNotAvailableInTargetError(symbol, target)
 
         return values
@@ -446,12 +464,12 @@ class Mapping(UIMapping):
             # we have a symbol: no type and code is fine
             return values
 
-        if is_this_a_macro(symbol):
+        if Parser.is_this_a_macro(symbol):
             # disallow output type and code for macros
             if type_ is not None or code is not None:
                 raise MacroButTypeOrCodeSetError()
 
-        if code is not None and code != system_mapping.get(symbol) or type_ != EV_KEY:
+        if code is not None and code != keyboard_layout.get(symbol) or type_ != EV_KEY:
             raise SymbolAndCodeMismatchError(symbol, code)
         return values
 
@@ -461,17 +479,27 @@ class Mapping(UIMapping):
         And vice versa."""
         assert isinstance(values.get("input_combination"), InputCombination)
         combination: InputCombination = values["input_combination"]
+
         analog_input_config = combination.find_analog_input_config()
-        use_as_analog = analog_input_config is not None
-
+        defines_analog_input = analog_input_config is not None
         output_type = values.get("output_type")
+        output_code = values.get("output_code")
+        mapping_type = values.get("mapping_type")
         output_symbol = values.get("output_symbol")
+        output_key_set = output_symbol or (output_type == EV_KEY and output_code)
 
-        if not use_as_analog and not output_symbol and output_type != EV_KEY:
+        if mapping_type is None:
+            # Empty mapping most likely
+            return values
+
+        if not defines_analog_input and mapping_type != MappingType.KEY_MACRO.value:
+            raise WrongMappingTypeForKeyError()
+
+        if not defines_analog_input and not output_key_set:
             raise MissingMacroOrKeyError()
 
         if (
-            use_as_analog
+            defines_analog_input
             and output_type not in (EV_ABS, EV_REL)
             and output_symbol != DISABLE_NAME
         ):
