@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -33,7 +33,7 @@ from typing import Dict, List, Optional, Tuple, Union
 
 import evdev
 
-from inputremapper.configs.input_config import InputCombination, InputConfig, DeviceHash
+from inputremapper.configs.input_config import InputCombination, InputConfig
 from inputremapper.configs.preset import Preset
 from inputremapper.groups import (
     _Group,
@@ -43,9 +43,10 @@ from inputremapper.groups import (
 from inputremapper.gui.messages.message_broker import MessageType
 from inputremapper.injection.context import Context
 from inputremapper.injection.event_reader import EventReader
+from inputremapper.injection.mapping_handlers.mapping_parser import MappingParser
 from inputremapper.injection.numlock import set_numlock, is_numlock_on, ensure_numlock
-from inputremapper.logger import logger
-from inputremapper.utils import get_device_hash
+from inputremapper.logging.logger import logger
+from inputremapper.utils import get_device_hash, DeviceHash
 
 CapabilitiesDict = Dict[int, List[int]]
 
@@ -61,7 +62,7 @@ class InjectorCommand(str, enum.Enum):
 class InjectorState(str, enum.Enum):
     UNKNOWN = "UNKNOWN"
     STARTING = "STARTING"
-    FAILED = "FAILED"
+    ERROR = "FAILED"
     RUNNING = "RUNNING"
     STOPPED = "STOPPED"
     NO_GRAB = "NO_GRAB"
@@ -119,7 +120,12 @@ class Injector(multiprocessing.Process):
 
     regrab_timeout = 0.2
 
-    def __init__(self, group: _Group, preset: Preset) -> None:
+    def __init__(
+        self,
+        group: _Group,
+        preset: Preset,
+        mapping_parser: MappingParser,
+    ) -> None:
         """
 
         Parameters
@@ -128,6 +134,7 @@ class Injector(multiprocessing.Process):
             the device group
         """
         self.group = group
+        self.mapping_parser = mapping_parser
         self._state = InjectorState.UNKNOWN
 
         # used to interact with the parts of this class that are running within
@@ -167,7 +174,7 @@ class Injector(multiprocessing.Process):
             if state in (InjectorState.STARTING, InjectorState.RUNNING) and not alive:
                 # we thought it is running (maybe it was when get_state was previously),
                 # but the process is not alive. It probably crashed
-                state = InjectorState.FAILED
+                state = InjectorState.ERROR
                 logger.error("Injector was unexpectedly found stopped")
 
         logger.debug(
@@ -340,19 +347,25 @@ class Injector(multiprocessing.Process):
             await frame_available.wait()
             frame_available.clear()
             msg = self._msg_pipe[0].recv()
-            if msg == InjectorCommand.CLOSE:
-                logger.debug("Received close signal")
-                self._stop_event.set()
-                # give the event pipeline some time to reset devices
-                # before shutting the loop down
-                await asyncio.sleep(0.1)
 
-                # stop the event loop and cause the process to reach its end
-                # cleanly. Using .terminate prevents coverage from working.
-                loop.stop()
-                self._msg_pipe[0].send(InjectorState.STOPPED)
+            if msg == InjectorCommand.CLOSE:
+                await self._close()
                 return
 
+    async def _close(self):
+        logger.debug("Received close signal")
+        self._stop_event.set()
+        # give the event pipeline some time to reset devices
+        # before shutting the loop down
+        await asyncio.sleep(0.1)
+
+        # stop the event loop and cause the process to reach its end
+        # cleanly. Using .terminate prevents coverage from working.
+        loop = asyncio.get_event_loop()
+        loop.stop()
+
+        self._msg_pipe[0].send(InjectorState.STOPPED)
+
     def _create_forwarding_device(self, source: evdev.InputDevice) -> evdev.UInput:
         # copy as much information as possible, because libinput uses the extra
         # information to enable certain features like "Disable touchpad while
@@ -402,8 +415,8 @@ class Injector(multiprocessing.Process):
 
         self._devices = self.group.get_devices()
 
-        # InputConfigs may not contain the origin_hash information, this will try to make a
-        # good guess if the origin_hash information is missing or invalid.
+        # InputConfigs may not contain the origin_hash information, this will try to
+        # make a good guess if the origin_hash information is missing or invalid.
         self._update_preset()
 
         # grab devices as early as possible. If events appear that won't get
@@ -415,7 +428,12 @@ class Injector(multiprocessing.Process):
 
         # create this within the process after the event loop creation,
         # so that the macros use the correct loop
-        self.context = Context(self.preset, sources, forward_devices)
+        self.context = Context(
+            self.preset,
+            sources,
+            forward_devices,
+            self.mapping_parser,
+        )
         self._stop_event = asyncio.Event()
 
         if len(sources) == 0:
