@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -26,7 +26,7 @@ from typing import Tuple, Optional, Hashable, Literal
 import evdev
 from evdev import ecodes
 
-from inputremapper.utils import get_evdev_constant_name
+from inputremapper.utils import get_evdev_constant_name, DeviceHash
 
 
 class EventActions(enum.Enum):
@@ -41,21 +41,6 @@ class EventActions(enum.Enum):
     negative_trigger = enum.auto()  # original event was negative direction
 
 
-def validate_event(event):
-    """Test if the event is valid."""
-    if not isinstance(event.type, int):
-        raise TypeError(f"Expected type to be an int, but got {event.type}")
-
-    if not isinstance(event.code, int):
-        raise TypeError(f"Expected code to be an int, but got {event.code}")
-
-    if not isinstance(event.value, int):
-        # this happened to me because I screwed stuff up
-        raise TypeError(f"Expected value to be an int, but got {event.value}")
-
-    return event
-
-
 # Todo: add slots=True as soon as python 3.10 is in common distros
 @dataclass(frozen=True)
 class InputEvent:
@@ -70,8 +55,7 @@ class InputEvent:
     code: int
     value: int
     actions: Tuple[EventActions, ...] = ()
-    origin_hash: Optional[str] = None
-    forward_to: Optional[evdev.UInput] = None
+    origin_hash: Optional[DeviceHash] = None
 
     def __eq__(self, other: InputEvent | evdev.InputEvent | Tuple[int, int, int]):
         # useful in tests
@@ -81,6 +65,21 @@ class InputEvent:
             return self.event_tuple == other
         raise TypeError(f"cannot compare {type(other)} with InputEvent")
 
+    @staticmethod
+    def validate_event(event):
+        """Test if the event is valid."""
+        if not isinstance(event.type, int):
+            raise TypeError(f"Expected type to be an int, but got {event.type}")
+
+        if not isinstance(event.code, int):
+            raise TypeError(f"Expected code to be an int, but got {event.code}")
+
+        if not isinstance(event.value, int):
+            # this happened to me because I screwed stuff up
+            raise TypeError(f"Expected value to be an int, but got {event.value}")
+
+        return event
+
     @property
     def input_match_hash(self) -> Hashable:
         """a Hashable object which is intended to match the InputEvent with a
@@ -90,7 +89,9 @@ class InputEvent:
 
     @classmethod
     def from_event(
-        cls, event: evdev.InputEvent, origin_hash: Optional[str] = None
+        cls,
+        event: evdev.InputEvent,
+        origin_hash: Optional[DeviceHash] = None,
     ) -> InputEvent:
         """Create a InputEvent from another InputEvent or evdev.InputEvent."""
         try:
@@ -109,7 +110,9 @@ class InputEvent:
 
     @classmethod
     def from_tuple(
-        cls, event_tuple: Tuple[int, int, int], origin_hash: Optional[str] = None
+        cls,
+        event_tuple: Tuple[int, int, int],
+        origin_hash: Optional[DeviceHash] = None,
     ) -> InputEvent:
         """Create a InputEvent from a (type, code, value) tuple."""
         # use this as rarely as possible. Construct objects early on and pass them
@@ -119,7 +122,7 @@ class InputEvent:
                 f"failed to create InputEvent {event_tuple = } must have length 3"
             )
 
-        return validate_event(
+        return cls.validate_event(
             cls(
                 0,
                 0,
@@ -131,9 +134,9 @@ class InputEvent:
         )
 
     @classmethod
-    def abs(cls, code: int, value: int, origin_hash: Optional[str] = None):
+    def abs(cls, code: int, value: int, origin_hash: Optional[DeviceHash] = None):
         """Create an abs event, like joystick movements."""
-        return validate_event(
+        return cls.validate_event(
             cls(
                 0,
                 0,
@@ -147,7 +150,7 @@ class InputEvent:
     @classmethod
     def rel(cls, code: int, value: int, origin_hash: Optional[str] = None):
         """Create a rel event, like mouse movements."""
-        return validate_event(
+        return cls.validate_event(
             cls(
                 0,
                 0,
@@ -164,7 +167,7 @@ class InputEvent:
 
         A value of 1 means "press", a value of 0 means "release".
         """
-        return validate_event(
+        return cls.validate_event(
             cls(
                 0,
                 0,
