@@ -1,7 +1,7 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -26,9 +26,8 @@ from unittest.mock import patch, MagicMock, call
 import gi
 from evdev.ecodes import EV_ABS, ABS_X, ABS_Y, ABS_RX
 
-from inputremapper.configs.system_mapping import system_mapping
+from inputremapper.configs.keyboard_layout import keyboard_layout
 from inputremapper.injection.injector import InjectorState
-from tests.lib.logger import logger
 
 gi.require_version("Gtk", "3.0")
 from gi.repository import Gtk
@@ -51,24 +50,25 @@ from inputremapper.gui.messages.message_data import (
 from inputremapper.gui.reader_client import ReaderClient
 from inputremapper.gui.utils import CTX_ERROR, CTX_APPLY, gtk_iteration
 from inputremapper.gui.gettext import _
-from inputremapper.injection.global_uinputs import GlobalUInputs
-from inputremapper.configs.mapping import UIMapping, MappingData, MappingType, Mapping
-from tests.lib.cleanup import quick_cleanup
-from tests.lib.stuff import spy
+from inputremapper.injection.global_uinputs import GlobalUInputs, FrontendUInput
+from inputremapper.configs.mapping import UIMapping, MappingData, Mapping
+from tests.lib.spy import spy
 from tests.lib.patches import FakeDaemonProxy
 from tests.lib.fixtures import fixtures, prepare_presets
 from inputremapper.configs.global_config import GlobalConfig
 from inputremapper.gui.controller import Controller, MAPPING_DEFAULTS
 from inputremapper.gui.data_manager import DataManager, DEFAULT_PRESET_NAME
-from inputremapper.configs.paths import get_preset_path, CONFIG_PATH
+from inputremapper.configs.paths import PathUtils
 from inputremapper.configs.preset import Preset
+from tests.lib.test_setup import test_setup
 
 
+@test_setup
 class TestController(unittest.TestCase):
     def setUp(self) -> None:
         super().setUp()
         self.message_broker = MessageBroker()
-        uinputs = GlobalUInputs()
+        uinputs = GlobalUInputs(FrontendUInput)
         uinputs.prepare_all()
         self.data_manager = DataManager(
             self.message_broker,
@@ -76,15 +76,12 @@ class TestController(unittest.TestCase):
             ReaderClient(self.message_broker, _Groups()),
             FakeDaemonProxy(),
             uinputs,
-            system_mapping,
+            keyboard_layout,
         )
         self.user_interface = MagicMock()
         self.controller = Controller(self.message_broker, self.data_manager)
         self.controller.set_gui(self.user_interface)
 
-    def tearDown(self) -> None:
-        quick_cleanup()
-
     def test_should_get_newest_group(self):
         """get_a_group should the newest group."""
         with patch.object(
@@ -138,7 +135,7 @@ class TestController(unittest.TestCase):
         self.message_broker.subscribe(MessageType.groups, f)
         self.message_broker.signal(MessageType.init)
         self.assertEqual(
-            ["Foo Device", "Foo Device 2", "Bar Device", "gamepad", "Qux/Device?"],
+            ["Foo Device", "Foo Device 2", "Bar Device", "gamepad", "Qux/[Device]?"],
             list(calls[-1].groups.keys()),
         )
 
@@ -231,7 +228,7 @@ class TestController(unittest.TestCase):
 
     def test_on_load_preset_should_provide_default_mapping(self):
         """If there is none."""
-        Preset(get_preset_path("Foo Device", "bar")).save()
+        Preset(PathUtils.get_preset_path("Foo Device", "bar")).save()
         self.data_manager.load_group("Foo Device 2")
         calls: List[MappingData] = []
 
@@ -253,7 +250,9 @@ class TestController(unittest.TestCase):
 
     def test_deletes_preset_when_confirmed(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
@@ -261,32 +260,46 @@ class TestController(unittest.TestCase):
             MessageType.user_confirm_request, lambda msg: msg.respond(True)
         )
         self.controller.delete_preset()
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "preset2")))
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
     def test_does_not_delete_preset_when_not_confirmed(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
         self.user_interface.confirm_delete.configure_mock(
             return_value=Gtk.ResponseType.CANCEL
         )
         self.controller.delete_preset()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
     def test_copy_preset(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
         self.controller.copy_preset()
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy"))
+        )
 
     def test_copy_preset_should_add_number(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
@@ -294,13 +307,21 @@ class TestController(unittest.TestCase):
         self.data_manager.load_preset("preset2")
         self.controller.copy_preset()  # creates "preset2 copy 2"
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy 2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy 2"))
+        )
 
     def test_copy_preset_should_increment_existing_number(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
@@ -310,27 +331,45 @@ class TestController(unittest.TestCase):
         self.data_manager.load_preset("preset2")
         self.controller.copy_preset()  # creates "preset2 copy 3"
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy 2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy 3")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy 2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy 3"))
+        )
 
     def test_copy_preset_should_not_append_copy_twice(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
         self.controller.copy_preset()  # creates "preset2 copy"
         self.controller.copy_preset()  # creates "preset2 copy 2" not "preset2 copy copy"
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy 2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy 2"))
+        )
 
     def test_copy_preset_should_not_append_copy_to_copy_with_number(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
@@ -339,99 +378,147 @@ class TestController(unittest.TestCase):
         self.controller.copy_preset()  # creates "preset2 copy 2"
         self.controller.copy_preset()  # creates "preset2 copy 3" not "preset2 copy 2 copy"
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy 2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2 copy 3")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy 2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 copy 3"))
+        )
 
     def test_rename_preset(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "foo")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertFalse(os.path.exists(PathUtils.get_preset_path("Foo Device", "foo")))
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
         self.controller.rename_preset(new_name="foo")
 
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "foo")))
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(os.path.exists(PathUtils.get_preset_path("Foo Device", "foo")))
 
     def test_rename_preset_sanitized(self):
-        Preset(get_preset_path("Qux/Device?", "bla")).save()
+        Preset(PathUtils.get_preset_path("Qux/[Device]?", "bla")).save()
 
-        self.assertTrue(os.path.isfile(get_preset_path("Qux/Device?", "bla")))
-        self.assertFalse(os.path.exists(get_preset_path("Qux/Device?", "blubb")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Qux/[Device]?", "bla"))
+        )
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Qux/[Device]?", "blubb"))
+        )
 
-        self.data_manager.load_group("Qux/Device?")
+        self.data_manager.load_group("Qux/[Device]?")
         self.data_manager.load_preset("bla")
         self.controller.rename_preset(new_name="foo:/bar")
 
         # all functions expect the true name, which is also shown to the user, but on
         # the file system it always uses sanitized names.
-        self.assertTrue(os.path.exists(get_preset_path("Qux/Device?", "foo__bar")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Qux/[Device]?", "foo__bar"))
+        )
 
         # since the name is never stored in an un-sanitized way, this can't work
-        self.assertFalse(os.path.exists(get_preset_path("Qux/Device?", "foo:/bar")))
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Qux/[Device]?", "foo:/bar"))
+        )
 
-        path = os.path.join(CONFIG_PATH, "presets", "Qux_Device_", "foo__bar.json")
+        path = os.path.join(
+            PathUtils.config_path(), "presets", "Qux_[Device]_", "foo__bar.json"
+        )
         self.assertTrue(os.path.exists(path))
 
         # using the sanitized name in function calls works as well
-        self.assertTrue(os.path.isfile(get_preset_path("Qux_Device_", "foo__bar")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Qux_[Device]_", "foo__bar"))
+        )
 
     def test_rename_preset_should_pick_available_name(self):
         prepare_presets()
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset3")))
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "preset3 2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3"))
+        )
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3 2"))
+        )
 
         self.data_manager.load_group("Foo Device")
         self.data_manager.load_preset("preset2")
         self.controller.rename_preset(new_name="preset3")
 
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset3")))
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset3 2")))
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3"))
+        )
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset3 2"))
+        )
 
     def test_rename_preset_should_not_rename_to_empty_name(self):
         prepare_presets()
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device")
         self.data_manager.load_preset("preset2")
         self.controller.rename_preset(new_name="")
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
     def test_rename_preset_should_not_update_same_name(self):
         """When the new name is the same as the current name."""
         prepare_presets()
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
         self.controller.rename_preset(new_name="preset2")
 
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "preset2")))
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "preset2 2")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
+        self.assertFalse(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "preset2 2"))
+        )
 
     def test_on_add_preset_uses_default_name(self):
         self.assertFalse(
-            os.path.exists(get_preset_path("Foo Device", DEFAULT_PRESET_NAME))
+            os.path.exists(PathUtils.get_preset_path("Foo Device", DEFAULT_PRESET_NAME))
         )
 
         self.data_manager.load_group("Foo Device 2")
 
         self.controller.add_preset()
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "new preset")))
+        self.assertTrue(
+            os.path.exists(PathUtils.get_preset_path("Foo Device", "new preset"))
+        )
 
     def test_on_add_preset_uses_provided_name(self):
-        self.assertFalse(os.path.exists(get_preset_path("Foo Device", "foo")))
+        self.assertFalse(os.path.exists(PathUtils.get_preset_path("Foo Device", "foo")))
 
         self.data_manager.load_group("Foo Device 2")
 
         self.controller.add_preset(name="foo")
-        self.assertTrue(os.path.exists(get_preset_path("Foo Device", "foo")))
+        self.assertTrue(os.path.exists(PathUtils.get_preset_path("Foo Device", "foo")))
 
     def test_on_add_preset_shows_permission_error_status(self):
         self.data_manager.load_group("Foo Device 2")
@@ -513,7 +600,9 @@ class TestController(unittest.TestCase):
 
     def test_deletes_mapping_when_confirmed(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
@@ -524,7 +613,7 @@ class TestController(unittest.TestCase):
         self.controller.delete_mapping()
         self.controller.save()
 
-        preset = Preset(get_preset_path("Foo Device", "preset2"))
+        preset = Preset(PathUtils.get_preset_path("Foo Device", "preset2"))
         preset.load()
         self.assertIsNone(
             preset.get_mapping(InputCombination([InputConfig(type=1, code=3)]))
@@ -532,7 +621,9 @@ class TestController(unittest.TestCase):
 
     def test_does_not_delete_mapping_when_not_confirmed(self):
         prepare_presets()
-        self.assertTrue(os.path.isfile(get_preset_path("Foo Device", "preset2")))
+        self.assertTrue(
+            os.path.isfile(PathUtils.get_preset_path("Foo Device", "preset2"))
+        )
 
         self.data_manager.load_group("Foo Device 2")
         self.data_manager.load_preset("preset2")
@@ -544,7 +635,7 @@ class TestController(unittest.TestCase):
         self.controller.delete_mapping()
         self.controller.save()
 
-        preset = Preset(get_preset_path("Foo Device", "preset2"))
+        preset = Preset(PathUtils.get_preset_path("Foo Device", "preset2"))
         preset.load()
         self.assertIsNotNone(
             preset.get_mapping(InputCombination([InputConfig(type=1, code=3)]))
@@ -908,7 +999,7 @@ class TestController(unittest.TestCase):
             calls[-1],
             StatusData(
                 CTX_APPLY,
-                _("Failed to apply preset %s") % self.data_manager.active_preset.name,
+                _('Failed to apply preset "%s"') % self.data_manager.active_preset.name,
             ),
         )
 
@@ -957,17 +1048,17 @@ class TestController(unittest.TestCase):
 
         self.controller.start_injecting()
         gtk_iteration(50)
-        self.assertEqual(calls[-1].msg, _("Applied preset %s") % "preset2")
+        self.assertEqual(calls[-1].msg, _('Applied preset "%s"') % "preset2")
 
-        mock.return_value = InjectorState.FAILED
+        mock.return_value = InjectorState.ERROR
         self.controller.start_injecting()
         gtk_iteration(50)
-        self.assertEqual(calls[-1].msg, _("Failed to apply preset %s") % "preset2")
+        self.assertEqual(calls[-1].msg, _('Error applying preset "%s"') % "preset2")
 
         mock.return_value = InjectorState.NO_GRAB
         self.controller.start_injecting()
         gtk_iteration(50)
-        self.assertEqual(calls[-1].msg, "The device was not grabbed")
+        self.assertEqual(calls[-1].msg, _('Failed to apply preset "%s"') % "preset2")
 
         mock.return_value = InjectorState.UPGRADE_EVDEV
         self.controller.start_injecting()
