@@ -1,7 +1,7 @@
-#!/usr/bin/python3
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # input-remapper - GUI for device specific keyboard mappings
-# Copyright (C) 2023 sezanzeb <proxima@sezanzeb.de>
+# Copyright (C) 2025 sezanzeb <b8x45ygc9@mozmail.com>
 #
 # This file is part of input-remapper.
 #
@@ -21,7 +21,6 @@
 
 """See TestEventPipeline for more tests."""
 
-
 import asyncio
 import unittest
 from unittest.mock import MagicMock
@@ -40,34 +39,34 @@ from evdev.ecodes import (
     REL_WHEEL,
 )
 
-from inputremapper.injection.mapping_handlers.combination_handler import (
-    CombinationHandler,
-)
-
-from inputremapper.injection.mapping_handlers.rel_to_btn_handler import RelToBtnHandler
-
-from inputremapper.configs.mapping import Mapping, DEFAULT_REL_RATE
 from inputremapper.configs.input_config import InputCombination, InputConfig
-from inputremapper.injection.global_uinputs import global_uinputs
+from inputremapper.configs.mapping import Mapping, DEFAULT_REL_RATE, KnownUinput
+from inputremapper.injection.global_uinputs import GlobalUInputs, UInput
 from inputremapper.injection.mapping_handlers.abs_to_abs_handler import AbsToAbsHandler
 from inputremapper.injection.mapping_handlers.abs_to_btn_handler import AbsToBtnHandler
 from inputremapper.injection.mapping_handlers.abs_to_rel_handler import AbsToRelHandler
-from inputremapper.injection.mapping_handlers.rel_to_rel_handler import RelToRelHandler
 from inputremapper.injection.mapping_handlers.axis_switch_handler import (
     AxisSwitchHandler,
 )
+from inputremapper.injection.mapping_handlers.combination_handler import (
+    CombinationHandler,
+)
 from inputremapper.injection.mapping_handlers.hierarchy_handler import HierarchyHandler
 from inputremapper.injection.mapping_handlers.key_handler import KeyHandler
 from inputremapper.injection.mapping_handlers.macro_handler import MacroHandler
-from inputremapper.injection.mapping_handlers.mapping_handler import MappingHandler
+from inputremapper.injection.mapping_handlers.mapping_handler import (
+    MappingHandler,
+    InputEventHandler,
+)
 from inputremapper.injection.mapping_handlers.rel_to_abs_handler import RelToAbsHandler
+from inputremapper.injection.mapping_handlers.rel_to_btn_handler import RelToBtnHandler
+from inputremapper.injection.mapping_handlers.rel_to_rel_handler import RelToRelHandler
 from inputremapper.input_event import InputEvent, EventActions
-
 from tests.lib.cleanup import cleanup
-from tests.lib.logger import logger
-from tests.lib.patches import InputDevice
 from tests.lib.constants import MAX_ABS
 from tests.lib.fixtures import fixtures
+from tests.lib.patches import InputDevice
+from tests.lib.test_setup import test_setup
 
 
 class BaseTests:
@@ -90,6 +89,7 @@ class BaseTests:
         mock.reset.assert_called()
 
 
+@test_setup
 class TestAxisSwitchHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination(
@@ -98,6 +98,8 @@ class TestAxisSwitchHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 InputConfig(type=1, code=3),
             )
         )
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = AxisSwitchHandler(
             input_combination,
             Mapping(
@@ -107,14 +109,18 @@ class TestAxisSwitchHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 output_code=1,
             ),
             MagicMock(),
+            self.global_uinputs,
         )
 
 
+@test_setup
 class TestAbsToBtnHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination(
             [InputConfig(type=3, code=5, analog_threshold=10)]
         )
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = AbsToBtnHandler(
             input_combination,
             Mapping(
@@ -122,12 +128,16 @@ class TestAbsToBtnHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 target_uinput="mouse",
                 output_symbol="BTN_LEFT",
             ),
+            global_uinputs=self.global_uinputs,
         )
 
 
+@test_setup
 class TestAbsToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination([InputConfig(type=EV_ABS, code=ABS_X)])
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = AbsToAbsHandler(
             input_combination,
             Mapping(
@@ -136,6 +146,7 @@ class TestAbsToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 output_type=EV_ABS,
                 output_code=ABS_X,
             ),
+            global_uinputs=self.global_uinputs,
         )
 
     async def test_reset(self):
@@ -144,16 +155,19 @@ class TestAbsToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
             source=InputDevice("/dev/input/event15"),
         )
         self.handler.reset()
-        history = global_uinputs.get_uinput("gamepad").write_history
+        history = self.global_uinputs.get_uinput("gamepad").write_history
         self.assertEqual(
             history,
             [InputEvent.from_tuple((3, 0, MAX_ABS)), InputEvent.from_tuple((3, 0, 0))],
         )
 
 
+@test_setup
 class TestRelToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination([InputConfig(type=EV_REL, code=REL_X)])
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = RelToAbsHandler(
             input_combination,
             Mapping(
@@ -162,6 +176,7 @@ class TestRelToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 output_type=EV_ABS,
                 output_code=ABS_X,
             ),
+            self.global_uinputs,
         )
 
     async def test_reset(self):
@@ -170,7 +185,7 @@ class TestRelToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
             source=InputDevice("/dev/input/event15"),
         )
         self.handler.reset()
-        history = global_uinputs.get_uinput("gamepad").write_history
+        history = self.global_uinputs.get_uinput("gamepad").write_history
         self.assertEqual(len(history), 2)
 
         # something large, doesn't matter
@@ -214,9 +229,12 @@ class TestRelToAbsHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
         self.assertEqual(self.handler._observed_rate, DEFAULT_REL_RATE)
 
 
+@test_setup
 class TestAbsToRelHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination([InputConfig(type=EV_ABS, code=ABS_X)])
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = AbsToRelHandler(
             input_combination,
             Mapping(
@@ -225,6 +243,7 @@ class TestAbsToRelHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 output_type=EV_REL,
                 output_code=REL_X,
             ),
+            self.global_uinputs,
         )
 
     async def test_reset(self):
@@ -236,12 +255,13 @@ class TestAbsToRelHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
         self.handler.reset()
         await asyncio.sleep(0.05)
 
-        count = global_uinputs.get_uinput("mouse").write_count
+        count = self.global_uinputs.get_uinput("mouse").write_count
         self.assertGreater(count, 6)  # count should be 60*0.2 = 12
         await asyncio.sleep(0.2)
-        self.assertEqual(count, global_uinputs.get_uinput("mouse").write_count)
+        self.assertEqual(count, self.global_uinputs.get_uinput("mouse").write_count)
 
 
+@test_setup
 class TestCombinationHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     handler: CombinationHandler
 
@@ -280,32 +300,38 @@ class TestCombinationHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
 
         self.context_mock = MagicMock()
 
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = CombinationHandler(
             input_combination,
             Mapping(
                 input_combination=input_combination.to_config(),
                 target_uinput="mouse",
                 output_symbol="BTN_LEFT",
+                release_combination_keys=True,
             ),
             self.context_mock,
+            global_uinputs=self.global_uinputs,
         )
 
+        sub_handler_mock = MagicMock(InputEventHandler)
+        self.handler.set_sub_handler(sub_handler_mock)
+
+        # insert our own test-uinput to see what is being written to it
+        self.uinputs = {
+            self.mouse_hash: evdev.UInput(),
+            self.keyboard_hash: evdev.UInput(),
+            self.gamepad_hash: evdev.UInput(),
+        }
+        self.context_mock.get_forward_uinput = lambda origin_hash: self.uinputs[
+            origin_hash
+        ]
+
     def test_forward_correctly(self):
         # In the past, if a mapping has inputs from two different sub devices, it
         # always failed to send the release events to the correct one.
         # Nowadays, self._context.get_forward_uinput(origin_hash) is used to
         # release them correctly.
-        mock = MagicMock()
-        self.handler.set_sub_handler(mock)
-
-        # insert our own test-uinput to see what is being written to it
-        uinputs = {
-            self.mouse_hash: evdev.UInput(),
-            self.keyboard_hash: evdev.UInput(),
-            self.gamepad_hash: evdev.UInput(),
-        }
-        self.context_mock.get_forward_uinput = lambda origin_hash: uinputs[origin_hash]
-
         # 1. trigger the combination
         self.handler.notify(
             InputEvent.rel(
@@ -335,30 +361,23 @@ class TestCombinationHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
         # 2. expect release events to be written to the correct devices, as indicated
         # by the origin_hash of the InputConfigs
         self.assertListEqual(
-            uinputs[self.mouse_hash].write_history,
+            self.uinputs[self.mouse_hash].write_history,
             [InputEvent.rel(self.input_combination[0].code, 0)],
         )
         self.assertListEqual(
-            uinputs[self.keyboard_hash].write_history,
+            self.uinputs[self.keyboard_hash].write_history,
             [InputEvent.key(self.input_combination[1].code, 0)],
         )
-        self.assertListEqual(
-            uinputs[self.gamepad_hash].write_history,
-            [InputEvent.key(self.input_combination[2].code, 0)],
-        )
+
+        # We do not expect a release event for this, because there was no key-down
+        # event when the combination triggered.
+        # self.assertListEqual(
+        #     self.uinputs[self.gamepad_hash].write_history,
+        #     [InputEvent.key(self.input_combination[2].code, 0)],
+        # )
 
     def test_no_forwards(self):
         # if a combination is not triggered, nothing is released
-        mock = MagicMock()
-        self.handler.set_sub_handler(mock)
-
-        # insert our own test-uinput to see what is being written to it
-        uinputs = {
-            self.mouse_hash: evdev.UInput(),
-            self.keyboard_hash: evdev.UInput(),
-        }
-        self.context_mock.get_forward_uinput = lambda origin_hash: uinputs[origin_hash]
-
         # 1. inject any two events
         self.handler.notify(
             InputEvent.rel(
@@ -378,18 +397,22 @@ class TestCombinationHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
         )
 
         # 2. expect no release events to be written
-        self.assertListEqual(uinputs[self.mouse_hash].write_history, [])
-        self.assertListEqual(uinputs[self.keyboard_hash].write_history, [])
+        self.assertListEqual(self.uinputs[self.mouse_hash].write_history, [])
+        self.assertListEqual(self.uinputs[self.keyboard_hash].write_history, [])
 
 
+@test_setup
 class TestHierarchyHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         self.mock1 = MagicMock()
         self.mock2 = MagicMock()
         self.mock3 = MagicMock()
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = HierarchyHandler(
             [self.mock1, self.mock2, self.mock3],
             InputConfig(type=EV_KEY, code=KEY_A),
+            self.global_uinputs,
         )
 
     def test_reset(self):
@@ -399,6 +422,7 @@ class TestHierarchyHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
         self.mock3.reset.assert_called()
 
 
+@test_setup
 class TestKeyHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination(
@@ -407,6 +431,8 @@ class TestKeyHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 InputConfig(type=1, code=3),
             )
         )
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = KeyHandler(
             input_combination,
             Mapping(
@@ -414,6 +440,7 @@ class TestKeyHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 target_uinput="mouse",
                 output_symbol="BTN_LEFT",
             ),
+            self.global_uinputs,
         )
 
     def test_reset(self):
@@ -421,60 +448,94 @@ class TestKeyHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
             InputEvent(0, 0, EV_REL, REL_X, 1, actions=(EventActions.as_key,)),
             source=InputDevice("/dev/input/event11"),
         )
-        history = global_uinputs.get_uinput("mouse").write_history
+        history = self.global_uinputs.get_uinput("mouse").write_history
         self.assertEqual(history[0], InputEvent.key(BTN_LEFT, 1))
         self.assertEqual(len(history), 1)
 
         self.handler.reset()
-        history = global_uinputs.get_uinput("mouse").write_history
+        history = self.global_uinputs.get_uinput("mouse").write_history
         self.assertEqual(history[1], InputEvent.key(BTN_LEFT, 0))
         self.assertEqual(len(history), 2)
 
 
+@test_setup
 class TestMacroHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
-        input_combination = InputCombination(
+        self.input_combination = InputCombination(
             (
                 InputConfig(type=2, code=0, analog_threshold=10),
                 InputConfig(type=1, code=3),
             )
         )
         self.context_mock = MagicMock()
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
+        self.set_handler(KnownUinput.KEYBOARD, "key(a)")
+
+    def set_handler(self, target_uinput: KnownUinput, macro: str):
         self.handler = MacroHandler(
-            input_combination,
+            self.input_combination,
             Mapping(
-                input_combination=input_combination.to_config(),
-                target_uinput="mouse",
-                output_symbol="hold_keys(BTN_LEFT, BTN_RIGHT)",
+                input_combination=self.input_combination.to_config(),
+                target_uinput=target_uinput,
+                output_symbol=macro,
             ),
             context=self.context_mock,
+            global_uinputs=self.global_uinputs,
         )
 
     async def test_reset(self):
+        self.set_handler(KnownUinput.MOUSE, "hold_keys(BTN_LEFT, BTN_RIGHT)")
+
         self.handler.notify(
             InputEvent(0, 0, EV_REL, REL_X, 1, actions=(EventActions.as_key,)),
             source=InputDevice("/dev/input/event11"),
         )
 
         await asyncio.sleep(0.1)
-        history = global_uinputs.get_uinput("mouse").write_history
+        history = self.global_uinputs.get_uinput(KnownUinput.MOUSE).write_history
         self.assertIn(InputEvent.key(BTN_LEFT, 1), history)
         self.assertIn(InputEvent.key(BTN_RIGHT, 1), history)
         self.assertEqual(len(history), 2)
 
         self.handler.reset()
         await asyncio.sleep(0.1)
-        history = global_uinputs.get_uinput("mouse").write_history
+        history = self.global_uinputs.get_uinput(KnownUinput.MOUSE).write_history
         self.assertIn(InputEvent.key(BTN_LEFT, 0), history[-2:])
         self.assertIn(InputEvent.key(BTN_RIGHT, 0), history[-2:])
         self.assertEqual(len(history), 4)
 
+    async def test_reset_output(self):
+        self.set_handler(KnownUinput.KEYBOARD, "key_down(a)")
 
+        history = self.global_uinputs.get_uinput(KnownUinput.KEYBOARD).write_history
+
+        self.handler.reset()
+        await asyncio.sleep(0.1)
+        self.assertEqual(len(history), 0)
+
+        self.handler.notify(
+            InputEvent(0, 0, EV_REL, REL_X, 1, actions=(EventActions.as_key,)),
+            source=InputDevice("/dev/input/event11"),
+        )
+        await asyncio.sleep(0.1)
+        self.assertEqual(len(history), 1)
+        self.assertIn(InputEvent.key(KEY_A, 1), history)
+
+        self.handler.reset()
+        await asyncio.sleep(0.1)
+        self.assertEqual(len(history), 2)
+        self.assertIn(InputEvent.key(KEY_A, 0), history)
+
+
+@test_setup
 class TestRelToBtnHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
     def setUp(self):
         input_combination = InputCombination(
             [InputConfig(type=2, code=0, analog_threshold=10)]
         )
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = RelToBtnHandler(
             input_combination,
             Mapping(
@@ -482,14 +543,18 @@ class TestRelToBtnHandler(BaseTests, unittest.IsolatedAsyncioTestCase):
                 target_uinput="mouse",
                 output_symbol="BTN_LEFT",
             ),
+            self.global_uinputs,
         )
 
 
+@test_setup
 class TestRelToRelHanlder(BaseTests, unittest.IsolatedAsyncioTestCase):
     handler: RelToRelHandler
 
     def setUp(self):
         input_combination = InputCombination([InputConfig(type=EV_REL, code=REL_X)])
+        self.global_uinputs = GlobalUInputs(UInput)
+        self.global_uinputs.prepare_all()
         self.handler = RelToRelHandler(
             input_combination,
             Mapping(
@@ -499,6 +564,7 @@ class TestRelToRelHanlder(BaseTests, unittest.IsolatedAsyncioTestCase):
                 output_value=20,
                 target_uinput="mouse",
             ),
+            self.global_uinputs,
         )
 
     def test_should_map(self):
